SystemOrganization addCategory: #'OmniST-Languages-LRSTM-Errors'!
SystemOrganization addCategory: #'OmniST-Languages-LRSTM-Model'!
SystemOrganization addCategory: #'OmniST-Languages-LRSTM-Tests'!
SystemOrganization addCategory: #'OmniST-Languages-LRSTM-Benchmarks'!
SystemOrganization addCategory: #'OmniST-Languages-POCs'!
SystemOrganization addCategory: #'OmniST-Languages-ActiveObject-Reflection'!
SystemOrganization addCategory: #'OmniST-Languages-AmbientTalk'!
SystemOrganization addCategory: #'OmniST-Languages-AmbientTalk-Reflection'!
SystemOrganization addCategory: #'OmniST-Languages-Clojure-Reflection'!
SystemOrganization addCategory: #'OmniST-Languages-STM-Reflection'!
SystemOrganization addCategory: #'OmniST-Languages-STM'!
SystemOrganization addCategory: #'OmniST-Languages-ActiveObject'!
SystemOrganization addCategory: #'OmniST-Languages-Clojure'!
SystemOrganization addCategory: #'OmniST-Languages-STM-Misc'!
SystemOrganization addCategory: #'OmniST-Languages-AmbientTalk-Tests'!

SMarkLoops subclass: #LRSTMBenchLoops
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Benchmarks'!

!LRSTMBenchLoops methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/4/2011 12:04'!
benchAtomicFloatLoop
	[ self benchFloatLoop ] atomic.! !

!LRSTMBenchLoops methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/4/2011 12:04'!
benchAtomicIntLoop
	[ self benchIntLoop ] atomic.! !

OstDomain subclass: #ATActor
	instanceVariableNames: 'process mailbox terminate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-AmbientTalk-Reflection'!

!ATActor class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 11:46'!
example
	| hello |
	hello := ATHelloWorld new.
	hello sayHello.
	hello saySomething: 'baz'.
	
	hello := ATActor newWith: [ ATHelloWorld new ].
	hello sayHello.
	hello saySomething: 'bar'.
! !

!ATActor class methodsFor: 'constructor' stamp: 'StefanMarr 1/3/2012 12:50'!
newWithInitializer: aBlock
	"Normally, we would return a reference to the domain,
	 but this is not really the way AmbientTalk works,
	 so, here we also return the value returned by the aBlock."
	| domain valueMsg promise |
	domain := self new.
	promise := domain evaluateAsync: aBlock.
	^ promise read.! !

!ATActor methodsFor: 'actor semantics' stamp: 'StefanMarr 1/2/2012 16:50'!
evaluateAsync: aBlock
	"This is an implementation level method.
	 The block is by convention not referring to
	 unsafe shared state."
	| msg |
	msg := ATMessage new.
	msg receiver: aBlock.
	msg setSelector: #value arguments: #().

	mailbox nextPut: msg.
	
	^ msg promise.! !

!ATActor methodsFor: 'initialization' stamp: 'StefanMarr 11/6/2011 21:57'!
initialize
	super initialize.
	
	mailbox := SharedQueue new.
	terminate := false.
	
	process := self spawnHere: [
		[terminate] whileFalse: [
			self processIncomingMessages.
		]].! !

!ATActor methodsFor: 'actor semantics' stamp: 'StefanMarr 1/3/2012 12:46'!
processIncomingMessages
	| msg result |
	Transcript log: 'processIncomingMessages'.
	msg := mailbox next.
	
	Transcript log: 'processIncomingMessages: Got message: ', msg selector asString.

	result := msg send.
	
	Transcript log: 'processIncomingMessages: applied method'.
	
	msg result: result.! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 1/2/2012 18:56'!
readField: idx of: obj
	"We rely on the semantics of Smalltalk here.
	 All methods are executed in the AT domain."
	^ obj instVarAt: idx! !

!ATActor methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 14:45'!
requestExecutionOf: aSelector on: anObj
	"SEE #requestExecutionOf:on:with: for the template implementation"
	| msg |
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: #().! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 1/3/2012 10:08'!
requestExecutionOf: aSelector on: anObj with: p1
	"All messages sent to objects inside another AmbientTalk domain
	 have to be executed asynchronously, by default,
	 all messages sent inside an actor/domain are executed
	 synchronously. (sync-execution results in direct returning of the result, no future/promise)
	
	 In AT, they do normally return nil, but here we do return always
	 the promise/future. I think, this is more consistent, especially
	 with the way how I handle creation of actors/domains.
	 
	 But, to be able to have simple stuff without promises returned,
	 we use the <noReturnValue> pragma."
	| msg currentDomain |
	currentDomain := OstDomain currentDomain.
	
	self == currentDomain ifTrue: [
		Transcript log: 'Execute synchronously: ', aSelector.
		^ anObj perform: aSelector with: p1.
	].
	
	Transcript log: 'requestExecutionOf: execute asynchronously: ', aSelector.
		
	msg := ATMessage new.
	msg setSelector: aSelector arguments: { p1 }.
	msg receiver: anObj.
	mailbox nextPut: msg.
	
	"WARNING: we do not support #noReturnValue at the moment, STEFAN 2012-01-02."
	^ msg result.
	
	"Transcript log: 'message sent. Returns promise: ', 
		(aCompiledMethod pragmas contains: [:p | p keyword = #noReturnValue])
			not asString."
	
	"(aCompiledMethod pragmas contains: [:p | p keyword = #noReturnValue])
		ifTrue:  [^ nil ]
		ifFalse: [^ msg promise]."
! !

!ATActor methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 14:45'!
requestExecutionOf: aSelector on: anObj with: p1 with: p2
	"SEE #requestExecutionOf:on:with: for the template implementation"
	| msg |
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2 }.! !

!ATActor methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 14:45'!
requestExecutionOf: aSelector on: anObj with: p1 with: p2 with: p3
	"SEE #requestExecutionOf:on:with: for the template implementation"
	| msg |
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3 }.! !

!ATActor methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 14:46'!
requestExecutionOf: aSelector on: anObj with: p1 with: p2 with: p3 with: p4
	"SEE #requestExecutionOf:on:with: for the template implementation"
	| msg |
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4 }.! !

!ATActor methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 14:46'!
requestExecutionOf: aSelector on: anObj with: p1 with: p2 with: p3 with: p4 with: p5
	"SEE #requestExecutionOf:on:with: for the template implementation"
	| msg |
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4. p5 }.! !

!ATActor methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 14:46'!
requestExecutionOf: aSelector on: anObj with: p1 with: p2 with: p3 with: p4 with: p5 with: p6
	"SEE #requestExecutionOf:on:with: for the template implementation"
	| msg |
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4. p5. p6 }.! !

!ATActor methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 14:48'!
requestExecutionOf: aSelector on: anObj with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7
	"SEE #requestExecutionOf:on:with: for the template implementation"
	| msg |
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4. p5. p6. p7 }.! !

!ATActor methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 14:48'!
requestExecutionOf: aSelector on: anObj with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8
	| msg |
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4. p5. p6. p7. p8 }.! !

!ATActor methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 14:47'!
requestExecutionOf: aSelector on: anObj with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9
	| msg |
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4. p5. p6. p7. p8. p9 }.! !

!ATActor methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 14:46'!
requestExecutionOf: aSelector on: anObj with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10
	"SEE #requestExecutionOf:on:with: for the template implementation"
	| msg |
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4. p5. p6. p7. p8. p9. p10 }.! !

!ATActor methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 14:47'!
requestExecutionOf: aSelector on: anObj with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10 with: p11
	| msg |
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10 with: p11.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4. p5. p6. p7. p8. p9. p10. p11 }.! !

!ATActor methodsFor: 'actor semantics' stamp: 'StefanMarr 1/3/2012 14:44'!
sendAsAsyncMessage: aSelector to: anObj with: anArgArray
	| msg |
	msg := ATMessage new.
	msg setSelector: aSelector arguments: anArgArray.
	msg receiver: anObj.
	mailbox nextPut: msg.
	
	^ msg result.! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 1/2/2012 18:55'!
write: val toField: idx of: obj
	"We rely on Smalltalk's semantics here, this can not be called from another
	 domain since all messages are execute in the AT domain"
	obj instVarAt: idx put: val.
	^ val! !

OstDomain subclass: #ATIsolateDomain
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-AmbientTalk-Reflection'!

OstDomain subclass: #LRSTMDomain
	instanceVariableNames: ''
	classVariableNames: 'Domain'
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Model'!

!LRSTMDomain class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/11/2011 22:19'!
domain
	"We only need one domain for the STM system."
	Domain ifNil: [ Domain := self new. ].
	^ Domain! !

!LRSTMDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/11/2011 23:19'!
readField: idx of: obj
	<omniDoNotTransform>
	^ obj atomicInstVarAt: idx.! !

!LRSTMDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/11/2011 22:26'!
readGlobal: association
	<omniDoNotTransform>
	^ association atomicValue! !

!LRSTMDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/11/2011 23:19'!
write: val toField: idx of: obj
	<omniDoNotTransform>
	^ obj atomicInstVarAt: idx put: val.! !

!LRSTMDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/11/2011 22:40'!
writeGlobal: association value: val
	<omniDoNotTransform>
	^ association atomicValue: val! !

OstDomain subclass: #OPActiveObjectDomain
	instanceVariableNames: 'obj process mailbox terminate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-ActiveObject-Reflection'!

!OPActiveObjectDomain class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/5/2011 20:26'!
newFor: anObject
	| domain |
	domain := self new.
	domain object: anObject.
	^ domain.! !

!OPActiveObjectDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 21:04'!
domainForNewObjects
	"Active objects are created explicitly,
	 classes tell whether their objects are supposed to be active.
	 This happens during #initialize.
	 
	 However, #domainForNewObjects is determined during
	 #basicNew, and for ActiveObjects we just assume as the standard
	 case that all objects are passive.
	 For the moment that means, they are not part of any domain."
	^ nil! !

!OPActiveObjectDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/4/2011 17:01'!
initialize
	super initialize.
	
	mailbox := SharedQueue new.
	terminate := false.
	
	process := self spawnHere: [
		[terminate] whileFalse: [
			self processIncomingMessages.
		]].! !

!OPActiveObjectDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/5/2011 20:27'!
object: anObject
	obj := anObject! !

!OPActiveObjectDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 20:08'!
processIncomingMessages
	| msg result |
	Transcript log: 'processIncomingMessages'.
	msg := mailbox next.
	
	Transcript log: 'processIncomingMessages: Got message: ', msg method selector.

	result := msg method valueWithReceiver: msg receiver arguments: msg arguments.
	
	Transcript log: 'processIncomingMessages: applied method'.
	
	msg result: result.! !

!OPActiveObjectDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 11/4/2011 17:16'!
readField: idx of: ignoredObjUseKnownOwnerInstead
	"No strong encapsulation here.
	 Will just read the data and return it."
	^ obj instVarAt: idx! !

!OPActiveObjectDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 20:47'!
requestExecutionOf: aCompiledMethod on: anObj with: arguments
	"For the active object, we want to execute the message asynchronously.
	 But since we can't tell whether something is a void method,
	 we will always need to block on the promise and wait until
	 it is resolved.

	 Another detail: the handling of self sends is also done asynchronously.
    This can be changed, depending on what is desired, but going with this
    approach makes bugs and conceptual inconsitency easier to spot.
    For more complex applications, that is problably not a good idea.
   
    For instance, I ran into trouble with object allocation, and requesting
    the omniDomain from objects by this. Asking for an omniDomain is a reflective
    action and needs to be done via the domain object, which also acts as a mirror."
	| msg |
	Transcript log: 'requestExecutionOf:'.
	msg := OActiveObjectMessage method: aCompiledMethod on: anObj with: arguments.
	mailbox nextPut: msg.
	
	Transcript log: 'message sent. Immediate return: ', (aCompiledMethod pragmas contains: [:p | p keyword = #noReturnValue]) asString.
	
	(aCompiledMethod pragmas contains: [:p | p keyword = #noReturnValue])
		ifTrue:  [^ anObj ]
		ifFalse: [^ msg resolve]! !

!OPActiveObjectDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 11/4/2011 17:16'!
write: val toField: idx of: ignoredObjUseKnownOwnerInstead
	"No strong encapsulation here, will just set the field and return the value."
	obj instVarAt: idx put: val.
	^ val! !

OstDomain subclass: #OPAgentDomain
	instanceVariableNames: 'mailbox terminate process obj'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-Clojure-Reflection'!

!OPAgentDomain class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:34'!
newFor: anObject
	| domain |
	domain := self new.
	domain object: anObject.
	^ domain.! !

!OPAgentDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 21:46'!
domainForNewObjects
	"If I find a way to have these objects in an immutable domain, they should go there.
	 I could keep track of a epoch counter and based on that decide that they are modifiable during
	 the same epoch in which they were created!!
	TODO look into this"
	^ nil! !

!OPAgentDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:35'!
initialize
	super initialize.
	
	mailbox := SharedQueue new.
	terminate := false.
	
	process := self spawnHere: [
		[terminate] whileFalse: [
			self processIncomingMessages.
		]].! !

!OPAgentDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:36'!
object: anObject
	obj := anObject! !

!OPAgentDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:36'!
processIncomingMessages
	| msg result |
	Transcript log: 'processIncomingMessages'.
	msg := mailbox next.
	
	Transcript log: 'processIncomingMessages: Got message'.
	
	msg method valueWithReceiver: obj arguments: msg arguments.
	
	Transcript log: 'processIncomingMessages: applied method'.! !

!OPAgentDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:37'!
readField: idx of: ignoredObjUseKnownOwnerInstead
	"No strong encapsulation here.
	 Will just read the data and return it."
	^ obj instVarAt: idx! !

!OPAgentDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:45'!
requestExecutionOf: aCompiledMethod on: anObj with: arguments
	"For the agent, we want to execute all messages asynchronously.
	 Nothing is going to block, and we return here only self.
	
	 Note: this is a bit different than the usual Clojure agents, because we have objects
	       here, it is simple to defined a protocol on an agent.
	       But we also have the typical #send: aBlock, and #send:with: predefined
	       However, if more parameter are desired then a custom method can just be
	       added, since every method in the class has the Clojure (send) semantics."
	
	| msg |
	Transcript log: 'requestExecutionOf:'.
	msg := OPAgentMessage method: aCompiledMethod with: arguments.
	mailbox nextPut: msg.

	^ anObj! !

!OPAgentDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:47'!
write: val toField: idx of: ignoredObjUseKnownOwnerInstead
	"No strong encapsulation here, will just set the field and return the value."
	obj instVarAt: idx put: val.
	^ val! !

OstDomain subclass: #OPAtomDomain
	instanceVariableNames: 'obj mtx'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-Clojure-Reflection'!

!OPAtomDomain class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 01:42'!
newFor: anObject
	| domain |
	domain := self new.
	domain object: anObject.
	^ domain.! !

!OPAtomDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 21:46'!
domainForNewObjects
	"If I find a way to have these objects in an immutable domain, they should go there.
	 I could keep track of a epoch counter and based on that decide that they are modifiable during
	 the same epoch in which they were created!!
	TODO look into this"
	^ nil! !

!OPAtomDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 01:32'!
initialize
	mtx := Mutex new.! !

!OPAtomDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 01:42'!
object: o
	obj := o! !

!OPAtomDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 01:30'!
readField: idx of: ignoredObjUseKnownOwnerInstead
	"No strong encapsulation here.
	 Will just read the data and return it."
	^ obj instVarAt: idx! !

!OPAtomDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 01:46'!
requestExecutionOf: aCompiledMethod on: anObj with: arguments
	"For the active object, we want to execute the message asynchronously.
	 But since we can't tell whether something is a void method,
	 we will always need to block on the promise and wait until
	 it is resolved."
	| requiresMtx sel |
	Transcript log: 'requestExecutionOf:'.
	
	sel := aCompiledMethod selector.
	
	requiresMtx :=  (sel = #compareTo:andSet:) or: [sel = #reset:].
	
	requiresMtx
		ifTrue:  [ mtx critical: [
			^ aCompiledMethod valueWithReceiver: obj arguments: arguments.]]
		ifFalse: [
			^ aCompiledMethod valueWithReceiver: obj arguments: arguments.].! !

!OPAtomDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 01:46'!
write: val toField: idx of: ignoredObjUseKnownOwnerInstead
	"No strong encapsulation here, will just set the field and return the value."
	obj instVarAt: idx put: val.
	^ val! !

OstDomain subclass: #OPSTMDomain
	instanceVariableNames: 'mtx runningTransactions commitMtx'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-STM-Reflection'!

!OPSTMDomain class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 14:28'!
newWith: initializerBlock
	"Normally, we would return a reference to the domain,
	 but this is not really the way AmbientTalk works,
	 so, here we also return the value returned by the initializerBlock."
	| domain valueMsg promise |
	domain := self new.
	valueMsg := initializerBlock class >> #value.
	promise := domain requestExecutionOf: valueMsg on: initializerBlock with: #().
	^ promise read.! !

!OPSTMDomain methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 18:32'!
commitProtected: theCommitBlock
	^ commitMtx critical: theCommitBlock! !

!OPSTMDomain methodsFor: 'stm' stamp: 'StefanMarr 11/7/2011 15:22'!
currentTransaction
	| proc |
	proc := Processor thisProcess.
	mtx critical: [
		^ runningTransactions at: proc ifAbsent: [nil].
	].! !

!OPSTMDomain methodsFor: 'stm' stamp: 'StefanMarr 11/7/2011 15:21'!
currentTransaction: transaction
	| proc |
	proc := Processor thisProcess.
	mtx critical: [
		| parent |
		parent := runningTransactions at: proc ifAbsent: [nil].
		transaction parent: parent.
		runningTransactions at: proc put: transaction.
	].! !

!OPSTMDomain methodsFor: 'initialization' stamp: 'StefanMarr 11/7/2011 18:45'!
initialize
	super initialize.
	
	mtx := Mutex new.
	runningTransactions := IdentityDictionary new.
	
	commitMtx := Mutex new.! !

!OPSTMDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 11/7/2011 18:11'!
readField: idx of: obj
	" 1. The read has to look in the transaction's write buffer.
	  2. If that failed, it needs to look for the current value in the read buffer.
	  3. If that failed, it needs to look into the actual object."
	| transaction |
	transaction := self currentTransaction.
	transaction
		ifNil: [
			^ obj instVarAt: idx]
		ifNotNil: [
			^ transaction readField: idx of: obj.].
	
! !

!OPSTMDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 11/7/2011 15:24'!
requestExecutionOf: aCompiledMethod on: anObj with: arguments
	"My STM does not have any semantics with regard to messages at the moment.
	 So, lets just do inplace execution"
	^ aCompiledMethod valueWithReceiver: anObj arguments: arguments.! !

!OPSTMDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 11/7/2011 18:13'!
write: val toField: idx of: obj
	"Delegate to the current transaction"
	| transaction |
	transaction := self currentTransaction.
	transaction
		ifNil: 	[^obj instVarAt: idx put: val.]
		ifNotNil: [
			Transcript log: 'write', val asString, ' toField: ', idx asString, ' of: ', obj asString.
			^ self currentTransaction write: val toField: idx of: obj.].! !

OstDomain subclass: #OPSTMHello
	instanceVariableNames: 'omniDomain a b'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-STM'!

!OPSTMHello class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 18:09'!
example
	| stmDomain p1 p2 hello |
	stmDomain := OPSTMDomain new.
	hello := OPSTMHello new.
	
	"Since this main thread is not running inside the OPSTMDomain, which would be easy, too,
	 but is not happening here, we need to manually move the object into the domain."
	stmDomain adoptObject: hello.
	
	hello a: 'main'; b: 'main'.
	
	p1 := stmDomain spawnHere: [
		OPTransaction atomic: [
			hello a: 'p1'.
			Processor yield.
			hello b: 'p1'.
	]].

	p2 := stmDomain spawnHere: [
		OPTransaction atomic: [
			| b |
			b := hello b.
			Processor yield.
			hello b: 'p2'.
	]].! !

!OPSTMHello methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 14:40'!
a
	^ a asString! !

!OPSTMHello methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 14:40'!
a: obj
	a := obj asString! !

!OPSTMHello methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 18:18'!
b
	Transcript log: 'Read b: ', b asString.
	^ b asString! !

!OPSTMHello methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 18:14'!
b: obj
	Transcript log: 'Write b: ', obj asString.
	b := obj asString! !

OstDomain subclass: #OPTransaction
	instanceVariableNames: 'theOmniDomain parent wasAborted writeBuffer readBuffer'
	classVariableNames: 'NoValue'
	poolDictionaries: ''
	category: 'OmniST-Languages-STM-Reflection'!

!OPTransaction class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 17:41'!
NoValue
	^ NoValue! !

!OPTransaction class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/11/2011 18:41'!
atomic: aBlock
	| domain transaction result |
	domain := OstDomain currentDomain.
	
	"We just do it without a transaction if the current domain
	 is not transactional.
	 TODO: make sure that is a sane semantics, especially
	       when interacting with other domains.
	"
	(domain isKindOf: OPSTMDomain) ifFalse: [
		^ aBlock value].
	
	transaction := self new.
	transaction domain: domain.
	domain currentTransaction: transaction.
	
	"Any error that is not catched before this point is "
	result := aBlock ifError: [:err :rcvr |
		"TODO: make sure that this signals an error"
		transaction abortWith: err on: rcvr.].
	
	transaction aborted
		ifTrue: [Error new messageText: 'we should never reach this point for an aborted transaction.'; signal].
	
	"Will also issue an error in case of conflicts.
	 No automatic retry here either."
	transaction commit.
	
	^ result! !

!OPTransaction class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 17:23'!
initialize
	super initialize.
	NoValue := Object new.! !

!OPTransaction methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 18:46'!
abort
	wasAborted := true.
	Error new messageText: 'Transaction aborted.' ; signal! !

!OPTransaction methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 15:40'!
aborted
	^ wasAborted! !

!OPTransaction methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 22:53'!
commit
	theOmniDomain commitProtected: [
		readBuffer checkConsistencyWithObjectsOtherwise: [self abort].
		writeBuffer performWrites.
	].

	Transcript log: 'Completed transaction successfully'.! !

!OPTransaction methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 14:55'!
domain: domain
	" I use a different name for the instance variable here,
	  since I am not sure whether I want to omni-fy these objects,
	  probably not, since it feels like reflective objects.
	  "
	theOmniDomain := domain! !

!OPTransaction methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 17:53'!
initialize
	super initialize.
	
	wasAborted := false.
	
	readBuffer := OPSTMBuffer new.
	writeBuffer := OPSTMBuffer new.! !

!OPTransaction methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 15:39'!
parent: aTransaction
	parent := aTransaction! !

!OPTransaction methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 17:26'!
readField: idx of: obj
	" 1. The read has to look in the transaction's write buffer.
	  2. If that failed, it needs to look for the current value in the read buffer.
	  3. If that failed, it needs to look into the actual object."
	| val |
	val := writeBuffer for: obj instVarAt: idx.
	val == NoValue ifFalse: [^ val].
	
	val := readBuffer for: obj instVarAt: idx.
	val == NoValue ifFalse: [^ val].
	
	val := obj instVarAt: idx.
	
	readBuffer for: obj instVarAt: idx put: val.
	
	^ val.! !

!OPTransaction methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 18:44'!
write: val toField: idx of: obj
	"REM: I was thinking whether to read a previously unread value to guarantee consistency,
	      but after discussion with Joeri, I decided against it.
	 So:
	 1. put the written value in the write buffer.
	 2. keep fingers crossed ;)"
	writeBuffer for: obj instVarAt: idx put: val.
	^ val! !

!ContextPart methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/31/2007 15:49'!
workingCopy
	^ self! !

Message subclass: #ATMessage
	instanceVariableNames: 'promise receiverObj'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-AmbientTalk-Reflection'!

!ATMessage class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 22:52'!
method: aCompiledMethod on: anObj with: arguments
	| msg |
	msg := self new.
	msg method: aCompiledMethod.
	msg receiver: anObj.
	msg arguments: arguments.
	^ msg! !

!ATMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/2/2012 18:14'!
initialize
	super initialize.
	promise := OPromise new.
	execAsync := false.! !

!ATMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 22:53'!
promise
	^ promise! !

!ATMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 22:52'!
receiver
	^ receiverObj! !

!ATMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 22:52'!
receiver: anObj
	receiverObj := anObj! !

!ATMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/2/2012 16:52'!
result
	"Result is blocking on the promise.
	 Use #promise to obtain the promise directly."
	^ promise read! !

!ATMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 22:52'!
result: anObj
	promise deliver: anObj! !

!ATMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/2/2012 16:53'!
send
	"answer the result of sending this message to its receiver"

	^ receiverObj perform: selector withArguments: args! !

!WriteStream methodsFor: '*OmniST-Languages-LRSTM' stamp: 'StefanMarr 12/4/2011 22:17'!
atomicNextPut: anObject
	((collection class == ByteString) and: [
		anObject isCharacter and:[anObject isOctetCharacter not]]) ifTrue: [
			collection := (WideString from: collection).
			^self nextPut: anObject.
	].
	position >= writeLimit
		ifTrue: [^ self pastEndPut: anObject]
		ifFalse: 
			[position := position + 1.
			^collection at: position put: anObject]! !

OstObject subclass: #ATHelloWorld
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-AmbientTalk'!

!ATHelloWorld methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 12:41'!
sayHello
	Transcript log: 'Hello World from within an actor. Actor Domain:', omniDomain printString, '(', omniDomain hash asString, ')'.
	self saySomething: 'foooo'.! !

!ATHelloWorld methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 12:40'!
saySomething: aString
	Transcript log: 'Something: ', aString, '. Actor Domain:', omniDomain printString, '(', omniDomain hash asString, ')'.! !

BenchmarkGameSuite subclass: #LRSTMBenchmarkGameSuite
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Benchmarks'!

!LRSTMBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/4/2011 12:16'!
benchAtomicBinaryTrees
	[self benchBinaryTrees ] atomic.! !

!LRSTMBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/4/2011 12:17'!
benchAtomicChameleons
	[self benchChameleons ] atomic.! !

!LRSTMBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/4/2011 12:17'!
benchAtomicChameneosRedux
	[self benchChameneosRedux ] atomic.! !

!LRSTMBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/4/2011 12:17'!
benchAtomicFannkuchRedux
	[self benchFannkuchRedux ] atomic.! !

!LRSTMBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/4/2011 12:18'!
benchAtomicFasta
	[self benchFasta ] atomic.! !

!LRSTMBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/4/2011 12:18'!
benchAtomicNBody
	[self benchNBody ] atomic.! !

!Process methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/30/2007 12:09'!
currentTransaction
	^ currentTransaction! !

!Process methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/30/2007 12:09'!
currentTransaction: aTransaction
	currentTransaction := aTransaction! !

Error subclass: #LRSTMConflict
	instanceVariableNames: 'transaction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Errors'!

!LRSTMConflict methodsFor: 'accessing' stamp: 'lr 5/25/2007 09:04'!
transaction
	^ transaction! !

!LRSTMConflict methodsFor: 'accessing' stamp: 'lr 5/25/2007 09:04'!
transaction: aTransaction
	transaction := aTransaction! !

Error subclass: #LRSTMInvalidTransaction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Errors'!

!BlockContext methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 2/26/2008 18:16'!
atomic
	^ LRSTMTransaction new do: self! !

!BlockContext methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/25/2007 09:13'!
atomicIfConflict: aBlock
	^ LRSTMTransaction new do: self ifConflict: aBlock! !

!BlockContext methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/25/2007 09:14'!
atomicRetry
	^ LRSTMTransaction new retry: self! !

!ReadStream methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 6/7/2007 15:16'!
atomicNext
	^ position >= readLimit
		ifFalse: [ collection at: (position := position + 1) ]! !

!BlockClosure methodsFor: '*OmniST-Languages-LRSTM' stamp: 'StefanMarr 11/28/2011 20:58'!
atomic
	^ LRSTMTransaction new do: self! !

!BlockClosure methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/25/2007 09:14'!
atomicIfConflict: aBlock
	^ LRSTMTransaction new do: self ifConflict: aBlock! !

!BlockClosure methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/25/2007 09:15'!
atomicRetry
	^ LRSTMTransaction new retry: self! !

!LookupKey methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/25/2007 11:18'!
atomicValue
	^ self workingCopy value! !

!LookupKey methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/25/2007 11:18'!
atomicValue: anObject
	^ self workingCopy value: anObject! !

TestCase subclass: #ATTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-AmbientTalk-Tests'!

!ATTests methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 10:56'!
testActorAndDomain
	"Make sure that the actor is correctly associated with its domain"! !

!ATTests methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 11:47'!
testNewWithInitializer
	"For the real AmbientTalk feeling"
	| result |
	result := ATActor newWithInitializer: [ATHelloWorld new].
	
	self assert: (result isKindOf: ATHelloWorld).
	! !

!ATTests methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 11:50'!
testNewWithInitializerATST
	"For the real AmbientTalk feeling"
	| result |
	result := ATActor newWithInitializer: [ATHelloWorld new].
	
	self assert: (result isKindOf: ATHelloWorld).
	self assert: result class == ATFarReference.! !

!ATTests methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/3/2012 13:02'!
testNewWithInitializerOmni
	"For the real AmbientTalk feeling"
	| result domain |
	result := ATActor newWithInitializer: [ATHelloWorld new].
	
	self assert: (result isKindOf: ATHelloWorld) description: 'Result should be the blocks return value'.
	
	domain := OstDomain omniDomainFrom: result.
	self assert: domain notNil description: 'The domain of the ATHelloWorld object should have been properly initialized...'.
	self assert: (domain isKindOf: ATActor) description: '... to the ATActor object.'. ! !

TestCase subclass: #LRSTMOmniTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Tests'!

!LRSTMOmniTest methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/11/2011 22:51'!
testOmniDomain
	| domain |
	[ domain := OstDomain currentDomain. ] atomic.

	self deny: domain isNil.
	self assert: (domain isKindOf: LRSTMDomain).! !

TestCase subclass: #LRSTMTest
	instanceVariableNames: 'value array'
	classVariableNames: 'Value'
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Tests'!

LRSTMTest subclass: #LRSTMAbortTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Tests'!

!LRSTMAbortTest methodsFor: 'testing-abort' stamp: 'StefanMarr 11/30/2011 20:12'!
testAbortAccessor
	self assert: [ self value: true; abort: self value ] atomic.
	self assert: self value isNil! !

!LRSTMAbortTest methodsFor: 'testing-abort' stamp: 'lr 5/14/2007 09:11'!
testAbortInstance
	self assert: [ value := true. self abort: value ] atomic.
	self assert: value isNil! !

!LRSTMAbortTest methodsFor: 'testing-abort' stamp: 'lr 5/14/2007 09:12'!
testAbortLiteral
	self assert: [ Value := true. self abort: Value ] atomic.
	self assert: Value isNil! !

!LRSTMAbortTest methodsFor: 'testing-abort' stamp: 'lr 5/14/2007 09:12'!
testAbortTemp
	| temp |
	self assert: [ temp := true. self abort: temp ] atomic.
	self assert: temp! !

!LRSTMAbortTest methodsFor: 'testing-abort' stamp: 'lr 5/14/2007 09:12'!
testAbortVariable
	self assert: [ array at: 1 put: true. self abort: (array at: 1) ] atomic.
	self assert: (array at: 1) isNil! !

!LRSTMAbortTest methodsFor: 'testing-check' stamp: 'lr 6/8/2007 16:50'!
testCheckAccessor
	self assert: [ self value: true; checkpoint; abort: self value ] atomic.
	self assert: self value! !

!LRSTMAbortTest methodsFor: 'testing-check' stamp: 'lr 5/14/2007 09:13'!
testCheckInstance
	self assert: [ value := true. self checkpoint. self abort: value ] atomic.
	self assert: value! !

!LRSTMAbortTest methodsFor: 'testing-check' stamp: 'lr 5/14/2007 09:13'!
testCheckLiteral
	self assert: [ Value := true. self checkpoint. self abort: Value ] atomic.
	self assert: Value! !

!LRSTMAbortTest methodsFor: 'testing-check' stamp: 'lr 5/14/2007 09:13'!
testCheckTemp
	| temp |
	self assert: [ temp := true. self checkpoint. self abort: temp ] atomic.
	self assert: temp! !

!LRSTMAbortTest methodsFor: 'testing-check' stamp: 'lr 5/14/2007 09:13'!
testCheckVariable
	self assert: [ array at: 1 put: true. self checkpoint. self abort: (array at: 1) ] atomic.
	self assert: (array at: 1)! !

!LRSTMAbortTest methodsFor: 'testing-error' stamp: 'lr 5/14/2007 08:39'!
testErrorAccessor
	self 
		should: [ [ self value: true. self assert: self value. 1 / 0 ] atomic ]
		raise: ZeroDivide.
	self assert: (array at: 1) isNil! !

!LRSTMAbortTest methodsFor: 'testing-error' stamp: 'lr 5/14/2007 08:39'!
testErrorInstance
	self 
		should: [ [ value := true. self assert: value. 1 / 0 ] atomic ]
		raise: ZeroDivide.
	self assert: value isNil! !

!LRSTMAbortTest methodsFor: 'testing-error' stamp: 'lr 5/14/2007 08:39'!
testErrorLiteral
	self 
		should: [ [ Value := true. self assert: Value. 1 / 0 ] atomic ]
		raise: ZeroDivide.
	self assert: Value isNil! !

!LRSTMAbortTest methodsFor: 'testing-error' stamp: 'lr 5/14/2007 08:43'!
testErrorTemp
	| temp |
	self 
		should: [ [ temp := true. self assert: temp. 1 / 0 ] atomic ]
		raise: ZeroDivide.
	self assert: temp! !

!LRSTMAbortTest methodsFor: 'testing-error' stamp: 'lr 5/14/2007 08:39'!
testErrorVariable
	self 
		should: [ [ array at: 1 put: true. self assert: (array at: 1). 1 / 0 ] atomic ]
		raise: ZeroDivide.
	self assert: (array at: 1) isNil! !

LRSTMTest subclass: #LRSTMBasicTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Tests'!

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:18'!
testAccessor
	self assert: [ self value: true. self value ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:18'!
testAccessorRead
	self value: true.
	self assert: [ self value ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:18'!
testAccessorWrite
	[ self value: true ] atomic.
	self assert: self value! !

!LRSTMBasicTest methodsFor: 'testing-basic' stamp: 'lr 5/9/2007 14:56'!
testBasicContext
	self assert: [ thisContext home ] atomic == thisContext! !

!LRSTMBasicTest methodsFor: 'testing-basic' stamp: 'lr 5/9/2007 11:18'!
testBasicSelf
	self assert: [ self ] atomic == self! !

!LRSTMBasicTest methodsFor: 'testing-basic' stamp: 'lr 5/9/2007 11:18'!
testBasicSuper
	self assert: [ super ] atomic == self! !

!LRSTMBasicTest methodsFor: 'testing-inheritance' stamp: 'StefanMarr 12/5/2011 11:35'!
testFieldReads
	| h |
	h := LRSTMHello3 new.
	self assert: #a equals: [h a] atomic.
	self assert: #b equals: [h b] atomic.
	self assert: #c equals: [h c] atomic.
	self assert: #d equals: [h d] atomic.
	self assert: #e equals: [h e] atomic.
	self assert: #f equals: [h f] atomic.! !

!LRSTMBasicTest methodsFor: 'testing-inheritance' stamp: 'StefanMarr 12/5/2011 11:37'!
testFieldReadsMixedInherited
	| h |
	h := LRSTMHello2 new.
	self assert: #a equals: [h a] atomic.
	self assert: #b equals: [h b] atomic.
	self assert: #c equals: [h c] atomic.
	self assert: #d equals: [h d] atomic.! !

!LRSTMBasicTest methodsFor: 'testing-inheritance' stamp: 'StefanMarr 12/5/2011 12:28'!
testFieldWrite
	| h |
	h := LRSTMHello3 new.
	
	[h a: #A] atomic.
	self assert: #A equals: [h a] atomic.
	
	[h b: #B] atomic.
	self assert: #B equals: [h b] atomic.
	
	[h c: #C] atomic.
	self assert: #C equals: [h c] atomic.
	
	[h d: #D] atomic.
	self assert: #D equals: [h d] atomic.
	
	[h e: #E] atomic.
	self assert: #E equals: [h e] atomic.
	
	[h f: #F] atomic.
	self assert: #F equals: [h f] atomic.! !

!LRSTMBasicTest methodsFor: 'testing-inheritance' stamp: 'StefanMarr 12/5/2011 12:30'!
testFieldWriteMixedInherited
	| h |
	h := LRSTMHello2 new.
	
	[h a: #A] atomic.
	self assert: #A equals: [h a] atomic.
	
	[h b: #B] atomic.
	self assert: #B equals: [h b] atomic.
	
	[h c: #C] atomic.
	self assert: #C equals: [h c] atomic.
	
	[h d: #D] atomic.
	self assert: #D equals: [h d] atomic.! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testGlobal
	self assert: [ GlobalValue := true. GlobalValue ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testGlobalRead
	GlobalValue := true.
	self assert: [ GlobalValue ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testGlobalWrite
	[ GlobalValue := true ] atomic.
	self assert: GlobalValue! !

!LRSTMBasicTest methodsFor: 'testing-inlined' stamp: 'lr 5/9/2007 14:58'!
testInlinedAndOr
	self assert: [ true and: [ true ] ] atomic.
	self deny: [ false and: [ true ] ] atomic.
	self deny: [ true and: [ false ] ] atomic.
	self deny: [ false and: [ false ] ] atomic.

	self assert: [ true or: [ true ] ] atomic.
	self assert: [ false or: [ true ] ] atomic.
	self assert: [ true or: [ false ] ] atomic.
	self deny: [ false and: [ false ] ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-inlined' stamp: 'lr 5/11/2007 15:07'!
testInlinedArray
	self assert: [ { } ] atomic = #( ).
	self assert: [ { 1 } ] atomic = #( 1 ).
	self assert: [ { 1. 2 } ] atomic = #( 1 2 ).
	self assert: [ { 1. 2. 3 } ] atomic = #( 1 2 3 ).
	self assert: [ { 1. 2. 3. 4 } ] atomic = #( 1 2 3 4 ).
	self assert: [ { 1. 2. 3. 4. 5 } ] atomic = #( 1 2 3 4 5 ).
	self assert: [ { 1. 2. 3. 4. 5. 6 } ] atomic = #( 1 2 3 4 5 6 )! !

!LRSTMBasicTest methodsFor: 'testing-inlined' stamp: 'lr 5/11/2007 15:23'!
testInlinedArrayNested
	self assert: [ { { } } ] atomic = #( ( ) ).
	self assert: [ { { 1 } } ] atomic = #( ( 1 ) ).
	self assert: [ { { 1. 2 } } ] atomic = #( ( 1 2 ) ).
	self assert: [ { { 1. 2 }. { 3 } } ] atomic = #( ( 1 2 ) ( 3 ) ).
	self assert: [ { { 1. 2 }. { 3. 4 } } ] atomic = #( ( 1 2 ) ( 3 4 ) )! !

!LRSTMBasicTest methodsFor: 'testing-inlined' stamp: 'lr 5/9/2007 14:58'!
testInlinedIfNil
	self assert: [ nil ifNil: [ true ] ] atomic.
	self assert: [ 1 ifNotNil: [ true ] ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-inlined' stamp: 'lr 5/9/2007 14:58'!
testInlinedIfTrue
	self assert: [ true ifTrue: [ true ] ] atomic.
	self assert: [ false ifFalse: [ true ] ] atomic.
	self assert: [ true ifTrue: [ true ] ifFalse: [ false ] ] atomic.
	self assert: [ false ifFalse: [ true ] ifTrue: [ false ] ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-inlined' stamp: 'lr 5/9/2007 11:42'!
testInlinedToDo
	| x |
	x := 0.
	[ 1 to: 10 do: [ :i | x := x + i ] ] atomic.
	self assert: x = 55.
	
	x := 0.
	[ 1 to: 10 by: 2 do: [ :i | x := x + i ] ] atomic.
	self assert: x = 25 ! !

!LRSTMBasicTest methodsFor: 'testing-inlined' stamp: 'lr 5/9/2007 15:14'!
testInlinedWhile
	| i x | 
	i := 1. x := 0.
	[ [ x := x + i. i := i + 1. i <= 10 ] whileTrue ] atomic.
	self assert: x = 55.
	
	i := 1. x := 0.
	[ [ i <= 10 ] whileTrue: [ x := x + i. i := i + 1 ] ] atomic.
	self assert: x = 55.
	
	i := 1. x := 0.
	[ [ x := x + i. i := i + 1. i > 10 ] whileFalse ] atomic.
	self assert: x = 55.
	
	i := 1. x := 0.
	[ [ i > 10 ] whileFalse: [ x := x + i. i := i + 1 ] ] atomic.
	self assert: x = 55! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testInstance
	self assert: [ value := true. value ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testInstanceRead
	value := true.
	self assert: [ value ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testInstanceWrite
	[ value := true ] atomic.
	self assert: value! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testLiteral
	self assert: [ Value := true. Value ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testLiteralRead
	Value := true.
	self assert: [ Value ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testLiteralWrite
	[ Value := true ] atomic.
	self assert: Value! !

!LRSTMBasicTest methodsFor: 'testing' stamp: 'lr 5/10/2007 09:48'!
testLoop
	"This code showed some bug in the exception handling."
	
 	| current |
	[ current := SortedCollection.
	[ current == nil ] whileFalse: [
		current == Object ifTrue: [ ^ self ].
		current := current superclass ] ]
			value.
	self assert: current == Object! !

!LRSTMBasicTest methodsFor: 'testing-sends' stamp: 'lr 5/9/2007 11:24'!
testSendBinary
	self assert: [ 1 + 2 ] atomic = 3! !

!LRSTMBasicTest methodsFor: 'testing-sends' stamp: 'lr 5/9/2007 11:24'!
testSendKeyword
	self assert: [ 2 raisedTo: 3 ] atomic = 8! !

!LRSTMBasicTest methodsFor: 'testing-sends' stamp: 'lr 5/9/2007 11:24'!
testSendUnary
	self assert: [ 1 negated ] atomic = -1! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testTemp
	| temp |
	self assert: [ temp := true. temp ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testTempRead
	| temp |
	temp := true.
	self assert: [ temp ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testTempWrite
	| temp |
	[ temp := true ] atomic.
	self assert: temp! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testVariable
	self assert: [ array at: 1 put: true; at: 1 ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testVariableRead
	array at: 1 put: true.
	self assert: [ array at: 1 ] atomic! !

!LRSTMBasicTest methodsFor: 'testing-kinds' stamp: 'lr 5/9/2007 11:19'!
testVariableWrite
	[ array at: 1 put: true ] atomic.
	self assert: (array at: 1)! !

LRSTMTest subclass: #LRSTMCUTransactionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Tests'!

!LRSTMCUTransactionTest commentStamp: 'StefanMarr 11/28/2011 22:08' prior: 0!
This test suite is actually originating from the Cutie-Helvetia-Trans implementation.
It is ported back to the STM implementation Lukas Renggli implemented originally with Geppetto on Squeak 3.9!

!LRSTMCUTransactionTest methodsFor: 'private' stamp: 'lr 2/23/2009 15:55'!
atomicInstVarAt: anInteger
	^ self workingCopy instVarAt: anInteger! !

!LRSTMCUTransactionTest methodsFor: 'private' stamp: 'lr 2/23/2009 15:55'!
atomicInstVarAt: anInteger put: anObject
	^ self workingCopy instVarAt: anInteger put: anObject! !

!LRSTMCUTransactionTest methodsFor: 'testing-custom' stamp: 'StefanMarr 11/28/2011 22:25'!
conflict
	self transaction 
		addChange: (LRSTMCustomChange 
			onApply: [ value := true ]
			onConflictTest: [ true ])! !

!LRSTMCUTransactionTest methodsFor: 'testing-custom' stamp: 'StefanMarr 11/28/2011 22:21'!
sideeffect
	self transaction 
		addChange: (LRSTMCustomChange
			onApply: [ value := true ])! !

!LRSTMCUTransactionTest methodsFor: 'testing-abort' stamp: 'lr 2/23/2009 16:31'!
testAbortAccessor
	self assert: [ self value: true; abort: self value ] atomic.
	self assert: self value isNil! !

!LRSTMCUTransactionTest methodsFor: 'testing-abort' stamp: 'lr 2/23/2009 16:31'!
testAbortInstance
	self assert: [ value := true. self abort: value ] atomic.
	self assert: value isNil! !

!LRSTMCUTransactionTest methodsFor: 'testing-abort' stamp: 'lr 2/23/2009 16:31'!
testAbortTemp
	| temp |
	self assert: [ temp := true. self abort: temp ] atomic.
	self assert: temp! !

!LRSTMCUTransactionTest methodsFor: 'testing-kinds' stamp: 'lr 2/23/2009 16:31'!
testAccessor
	self assert: [ self value: true. self value ] atomic! !

!LRSTMCUTransactionTest methodsFor: 'testing-kinds' stamp: 'lr 2/23/2009 16:31'!
testAccessorRead
	self value: true.
	self assert: [ self value ] atomic! !

!LRSTMCUTransactionTest methodsFor: 'testing-kinds' stamp: 'lr 2/23/2009 16:31'!
testAccessorWrite
	[ self value: true ] atomic.
	self assert: self value! !

!LRSTMCUTransactionTest methodsFor: 'testing' stamp: 'lr 2/23/2009 16:31'!
testBasicContext
	self assert: [ thisContext home ] atomic == thisContext! !

!LRSTMCUTransactionTest methodsFor: 'testing' stamp: 'lr 2/23/2009 16:31'!
testBasicSelf
	self assert: [ self ] atomic == self! !

!LRSTMCUTransactionTest methodsFor: 'testing' stamp: 'lr 2/23/2009 16:31'!
testBasicSuper
	self assert: [ super ] atomic == self! !

!LRSTMCUTransactionTest methodsFor: 'testing-check' stamp: 'lr 2/23/2009 16:31'!
testCheckAccessor
	self assert: [ self value: true; checkpoint; abort: self value ] atomic.
	self assert: self value! !

!LRSTMCUTransactionTest methodsFor: 'testing-check' stamp: 'lr 2/23/2009 16:31'!
testCheckInstance
	self assert: [ value := true. self checkpoint. self abort: value ] atomic.
	self assert: value! !

!LRSTMCUTransactionTest methodsFor: 'testing-check' stamp: 'lr 2/23/2009 16:31'!
testCheckTemp
	| temp |
	self assert: [ temp := true. self checkpoint. self abort: temp ] atomic.
	self assert: temp! !

!LRSTMCUTransactionTest methodsFor: 'testing-custom' stamp: 'StefanMarr 11/28/2011 22:25'!
testConflict
	self 
		should: [ [ self conflict ] atomic ]
		raise: LRSTMConflict.
	self assert: value isNil! !

!LRSTMCUTransactionTest methodsFor: 'testing-error' stamp: 'lr 2/23/2009 16:31'!
testErrorAccessor
	self 
		should: [ [ self value: true. self assert: self value. 1 / 0 ] atomic ]
		raise: ZeroDivide.
	self assert: value isNil! !

!LRSTMCUTransactionTest methodsFor: 'testing-error' stamp: 'lr 2/23/2009 16:31'!
testErrorInstance
	self 
		should: [ [ value := true. self assert: value. 1 / 0 ] atomic ]
		raise: ZeroDivide.
	self assert: value isNil! !

!LRSTMCUTransactionTest methodsFor: 'testing-error' stamp: 'lr 2/23/2009 16:31'!
testErrorTemp
	| temp |
	self 
		should: [ [ temp := true. self assert: temp. 1 / 0 ] atomic ]
		raise: ZeroDivide.
	self assert: temp! !

!LRSTMCUTransactionTest methodsFor: 'testing-kinds' stamp: 'lr 2/23/2009 16:31'!
testInstance
	self assert: [ value := true. value ] atomic! !

!LRSTMCUTransactionTest methodsFor: 'testing-kinds' stamp: 'lr 2/23/2009 16:31'!
testInstanceRead
	value := true.
	self assert: [ value ] atomic! !

!LRSTMCUTransactionTest methodsFor: 'testing-kinds' stamp: 'lr 2/23/2009 16:31'!
testInstanceWrite
	[ value := true ] atomic.
	self assert: value! !

!LRSTMCUTransactionTest methodsFor: 'testing-custom' stamp: 'lr 2/23/2009 16:31'!
testSideeffect
	[ self sideeffect ] atomic.
	self assert: value! !

!LRSTMCUTransactionTest methodsFor: 'private' stamp: 'lr 2/23/2009 16:43'!
workingCopy
	"Answer a working copy to be used within the context of the current transaction."
	
	| transaction |
	transaction := self transaction
		ifNil: [ ^ self ].
	^ (transaction changeFor: self) 
		working! !

LRSTMTest subclass: #LRSTMCollectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Tests'!

!LRSTMCollectionTest methodsFor: 'testing' stamp: 'lr 5/9/2007 10:04'!
testArrayLong
	| arr |
	arr := Array new: 10 withAll: 1.
	[ 3 to: 10 do: [ :each |
		arr at: each put: (arr at: each - 1) + (arr at: each - 2) ] ]
			atomic.
	3 to: 10 do: [ :each |
		self assert: (arr at: each) - (arr at: each - 1) = (arr at: each - 2) ]! !

!LRSTMCollectionTest methodsFor: 'testing' stamp: 'lr 5/9/2007 10:04'!
testArrayShort
	| arr |
	arr := Array new: 10 withAll: 1.
	3 to: 10 do: [ :each |
		[ arr at: each put: (arr at: each - 1) + (arr at: each - 2) ]
			atomic ].
	3 to: 10 do: [ :each |
		self assert: (arr at: each) - (arr at: each - 1) = (arr at: each - 2) ]! !

!LRSTMCollectionTest methodsFor: 'testing' stamp: 'lr 5/9/2007 10:04'!
testBagLong
	| bag |
	bag := Bag new.
	[ 1 to: 10 do: [ :each |
		bag add: each; add: each ] ]
			atomic.
	self assert: bag size = 20.
	1 to: 10 do: [ :each | 
		self assert: (bag occurrencesOf: each) = 2 ]! !

!LRSTMCollectionTest methodsFor: 'testing' stamp: 'lr 5/10/2007 14:19'!
testBagShort
	| bag |
	bag := Bag new.
	1 to: 10 do: [ :each |
		[ bag add: each; add: each ]
			atomic ].
	self assert: bag size = 20.
	1 to: 10 do: [ :each | 
		self assert: (bag occurrencesOf: each) = 2 ]! !

!LRSTMCollectionTest methodsFor: 'testing' stamp: 'lr 5/9/2007 10:03'!
testDictionaryLong
	| dict |
	dict := Dictionary new.
	[ 1 to: 10 do: [ :each | 
		dict at: each put: each ] ]
			atomic.
	self assert: dict size = 10.
	1 to: 10 do: [ :each | 
		self assert: (dict at: each) = each ]! !

!LRSTMCollectionTest methodsFor: 'testing' stamp: 'lr 5/9/2007 10:03'!
testDictionaryShort
	| dict |
	dict := Dictionary new.
	1 to: 10 do: [ :each | 
		[ dict at: each put: each ]
			atomic ].
	self assert: dict size = 10.
	1 to: 10 do: [ :each | 
		self assert: (dict at: each) = each ]! !

!LRSTMCollectionTest methodsFor: 'testing-streams' stamp: 'lr 5/14/2007 15:44'!
testReadStream
	| stream |
	stream := (1 to: 10) asArray readStream.
	[ 1 to: 10 do: [ :each | self assert: stream next = each ] ]
		atomic! !

!LRSTMCollectionTest methodsFor: 'testing' stamp: 'lr 5/9/2007 10:04'!
testSetLong
	| set |
	set := Set new.
	[ 1 to: 10 do: [ :each | 
		set add: each ] ] 
			atomic.
	self assert: set size = 10.
	1 to: 10 do: [ :each | 
		self assert: (set includes: each) ]! !

!LRSTMCollectionTest methodsFor: 'testing' stamp: 'lr 5/9/2007 10:03'!
testSetShort
	| set |
	set := Set new.
	1 to: 10 do: [ :each |
		[ set add: each ]
			atomic ].
	self assert: set size = 10.
	1 to: 10 do: [ :each | 
		self assert: (set includes: each) ]! !

!LRSTMCollectionTest methodsFor: 'testing-streams' stamp: 'lr 5/14/2007 15:57'!
testStreamContents
	[ array := Array streamContents: [ :stream |
		1 to: 10 do: [ :each | 
			stream nextPut: each ] ] ]
		atomic.
	self assert: (1 to: 10) asArray = array! !

!LRSTMCollectionTest methodsFor: 'testing-streams' stamp: 'lr 5/14/2007 15:45'!
testWriteStream
	| stream |
	stream := (Array new: 10) writeStream.
	[ 1 to: 10 do: [ :each | stream nextPut: each ] ]
		atomic.
	self assert: (1 to: 10) asArray = stream contents! !

LRSTMTest subclass: #LRSTMCustomTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Tests'!

!LRSTMCustomTest methodsFor: 'actions' stamp: 'StefanMarr 12/11/2011 16:32'!
conflict
	<omniDoNotTransform>

	self transaction 
		addChange: (LRSTMCustomChange 
			onApply: [ value := true ]
			onConflictTest: [ true ])! !

!LRSTMCustomTest methodsFor: 'actions' stamp: 'StefanMarr 12/11/2011 16:35'!
sideeffect
	<omniDoNotTransform>

	self transaction 
		addChange: (LRSTMCustomChange onApply: [ value := true ])! !

!LRSTMCustomTest methodsFor: 'testing' stamp: 'lr 2/26/2008 16:22'!
testConflict
	self 
		should: [ [ self conflict ] atomic ]
		raise: LRSTMConflict.
	self assert: value isNil! !

!LRSTMCustomTest methodsFor: 'testing' stamp: 'lr 2/26/2008 16:23'!
testSideeffect
	[ self sideeffect ] atomic.
	self assert: value! !

LRSTMTest subclass: #LRSTMNestedTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Tests'!

!LRSTMNestedTest methodsFor: 'testing' stamp: 'lr 2/26/2008 18:46'!
testAbortInner
	self assert: [ value := true. [ value := false. self abort: value ] atomic. value ] atomic.
	self assert: value! !

!LRSTMNestedTest methodsFor: 'testing' stamp: 'lr 2/26/2008 18:50'!
testAbortOuter
	self assert: [ [ value := true ] atomic. self abort: value ] atomic.
	self assert: value isNil! !

!LRSTMNestedTest methodsFor: 'testing' stamp: 'lr 2/26/2008 16:54'!
testEmpty
	[ [ ] atomic ] atomic! !

!LRSTMNestedTest methodsFor: 'testing' stamp: 'lr 2/26/2008 18:46'!
testKind
	self assert: [ self transaction ] atomic isNested not.
	self assert: [ [ self transaction ] atomic ] atomic isNested.
	self assert: [ [ [ self transaction ] atomic ] atomic ] atomic isNested.
	
	self assert: [ [ ] atomic. self transaction ] atomic isNested not.! !

!LRSTMNestedTest methodsFor: 'testing' stamp: 'lr 2/26/2008 16:54'!
testSimple
	[ [ value := true ] atomic ] atomic.
	self assert: value! !

!LRSTMNestedTest methodsFor: 'testing' stamp: 'lr 2/26/2008 18:24'!
testValueInner
	self assert: [ value := true. [ value ] atomic ] atomic.
	self assert: value! !

!LRSTMNestedTest methodsFor: 'testing' stamp: 'lr 2/26/2008 18:24'!
testValueOuter
	self assert: [ [ value := true ] atomic. value ] atomic.
	self assert: value! !

!LRSTMTest class methodsFor: 'initialization' stamp: 'lr 5/4/2007 14:37'!
initialize
	self environment at: #GlobalValue put: nil! !

!LRSTMTest class methodsFor: 'testing' stamp: 'lr 2/26/2008 16:27'!
isAbstract
	^ self name = #LRSTMTest! !

!LRSTMTest methodsFor: 'utilities' stamp: 'StefanMarr 12/11/2011 16:28'!
abort: anObject
	<omniDoNotTransform>

	self transaction abort: anObject! !

!LRSTMTest methodsFor: 'utilities' stamp: 'StefanMarr 12/11/2011 16:32'!
checkpoint
	<omniDoNotTransform>

	self transaction checkpoint! !

!LRSTMTest methodsFor: 'running' stamp: 'lr 2/26/2008 18:30'!
setUp
	value := nil.
	array := Array new: 1.
	Value := nil.
	GlobalValue := nil! !

!LRSTMTest methodsFor: 'running' stamp: 'lr 2/26/2008 18:30'!
tearDown
	Processor activeProcess currentTransaction: nil! !

!LRSTMTest methodsFor: 'utilities' stamp: 'StefanMarr 12/11/2011 16:35'!
transaction
	<omniDoNotTransform>

	^ Processor activeProcess currentTransaction! !

!LRSTMTest methodsFor: 'accessing' stamp: 'lr 5/3/2007 15:59'!
value
	^ value! !

!LRSTMTest methodsFor: 'accessing' stamp: 'lr 6/8/2007 16:52'!
value: anObject
	value := anObject! !

Object subclass: #LRSTMBenchmark
	instanceVariableNames: ''
	classVariableNames: 'Value'
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Tests'!

LRSTMBenchmark class
	instanceVariableNames: 'value mutex doneCount'!

LRSTMBenchmark class
	instanceVariableNames: 'value mutex doneCount'!

!LRSTMBenchmark class methodsFor: 'utilities' stamp: 'lr 5/30/2007 16:10'!
benchmark: aOriginalBlock reference: aReferenceBlock label: aString
	"Assert that aFastBlock is executed faster than aSlow block. The two blocks should be relatively fast and have no side effects, as they are executed multiple times."
	
	| originalTime referenceTime |
	originalTime := aOriginalBlock timeToRun.
	referenceTime := aReferenceBlock timeToRun.
	Transcript 
		show: aString; tab;
		show: ((referenceTime / 1000.0) roundTo: 0.01); tab;
		show: ((originalTime / 1000.0) roundTo: 0.01); tab;
		show: ((originalTime / referenceTime) asFloat roundTo: 0.01); cr! !

!LRSTMBenchmark class methodsFor: 'benchmarks' stamp: 'lr 5/30/2007 16:16'!
benchmarkActivation
	"self benchmarkActivation"
	
	self 
		benchmark: [ 10000000 timesRepeat: [ [ ] atomic ] ]
		reference: [ 10000000 timesRepeat: [ [ ] value ] ]
		label: 'Activation'! !

!LRSTMBenchmark class methodsFor: 'benchmarks' stamp: 'lr 5/30/2007 16:17'!
benchmarkBinding
	"self benchmarkBinding"

	self
		benchmark: [ [ 10000000 timesRepeat: [ Value ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ Value ] ] value ]
		label: 'Binding Read'.
	self
		benchmark: [ [ 10000000 timesRepeat: [ Value := nil ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ Value := nil ] ] value ]
		label: 'Binding Write'.! !

!LRSTMBenchmark class methodsFor: 'benchmarks' stamp: 'lr 5/30/2007 16:17'!
benchmarkInstance
	"self benchmarkInstance"
	
	self 
		benchmark: [ [ 10000000 timesRepeat: [ value ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ value ] ] value ]
		label: 'Instance Read'.
	self
		benchmark: [ [ 10000000 timesRepeat: [ value := nil ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ value := nil ] ] value ]
		label: 'Instance Write'! !

!LRSTMBenchmark class methodsFor: 'btree' stamp: 'lr 5/31/2007 17:11'!
benchmarkParallel
	"self benchmarkParallel"

	| semaphore data doneCount repeatCount time |
	semaphore := Semaphore forMutualExclusion.
	
	1 to: 70 do: [ :processCount |
		Smalltalk garbageCollect.
		
		data := BTree new.
		doneCount := 0.
		repeatCount := 500.
	
		processCount timesRepeat: [
			[ repeatCount timesRepeat: [ self twoAtomicUpdates: data ].
			  semaphore critical: [ doneCount := doneCount + 1 ] ]
				forkAt: Processor activeProcess priority ].
		time := [
			[ doneCount < processCount ]
				whileTrue: [ Processor yield ] ]
			timeToRun.

		Transcript show: processCount; tab; show: time; cr ]! !

!LRSTMBenchmark class methodsFor: 'benchmarks' stamp: 'lr 5/30/2007 16:17'!
benchmarkSend
	"self benchmarkSend"
	
	self
		benchmark: [ [ 10000000 timesRepeat: [ nil flag: #zork ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ nil flag: #zork ] ] value ]
		label: 'Message invokation'.
	self
		benchmark: [ [ 10000000 timesRepeat: [ nil flag: #zork ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ 1 + 2 ] ] value ]
		label: 'Message invokation (fast)'! !

!LRSTMBenchmark class methodsFor: 'benchmarks' stamp: 'lr 5/30/2007 16:18'!
benchmarkVariable
	"self benchmarkVariable"

	self 
		benchmark: [ [ 10000000 timesRepeat: [ #( nil ) at: 1 ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ #( nil ) at: 1 ] ] value ]
		label: 'Variable Read'.
	self
		benchmark: [ [ 10000000 timesRepeat: [ #( nil ) at: 1 put: nil ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ #( nil ) at: 1 put: nil ] ] value ]
		label: 'Variable Write'! !

!LRSTMBenchmark class methodsFor: 'btree' stamp: 'lr 5/31/2007 17:11'!
twoAtomicUpdates: aDictionary
	[ [ aDictionary 
		at: SmallInteger maxVal atRandom put: 0;
		at: SmallInteger maxVal atRandom put: 0 ]
			atomic ]
		on: LRSTMConflict
		do: [ self twoAtomicUpdates: aDictionary ]! !

!LRSTMBenchmark class methodsFor: 'btree' stamp: 'lr 6/1/2007 11:01'!
twoMutexUpdates: aDictionary
	mutex critical: [
		aDictionary
			at: SmallInteger maxVal atRandom put: 0;
			at: SmallInteger maxVal atRandom put: 0 ]! !

Object subclass: #LRSTMChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Model'!

!LRSTMChange methodsFor: 'actions' stamp: 'lr 6/8/2007 16:59'!
apply
	self subclassResponsibility! !

!LRSTMChange methodsFor: 'testing' stamp: 'lr 6/8/2007 17:00'!
hasChanged
	^ true! !

!LRSTMChange methodsFor: 'testing' stamp: 'lr 6/8/2007 17:00'!
hasConflict
	^ false! !

LRSTMChange subclass: #LRSTMCustomChange
	instanceVariableNames: 'applyBlock conflictTestBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Model'!

!LRSTMCustomChange class methodsFor: 'instance-creation' stamp: 'StefanMarr 12/11/2011 16:33'!
onApply: anApplyBlock
	<omniDoNotTransform>
	^ self onApply: anApplyBlock onConflictTest: [ false ]! !

!LRSTMCustomChange class methodsFor: 'instance-creation' stamp: 'StefanMarr 12/11/2011 16:33'!
onApply: anApplyBlock onConflictTest: aConflictTestBlock
	<omniDoNotTransform>
	^ self basicNew initializeApply: anApplyBlock conflictTest: aConflictTestBlock! !

!LRSTMCustomChange methodsFor: 'actions' stamp: 'lr 2/26/2008 16:26'!
apply
	applyBlock value! !

!LRSTMCustomChange methodsFor: 'testing' stamp: 'lr 6/8/2007 17:28'!
hasConflict
	^ conflictTestBlock value! !

!LRSTMCustomChange methodsFor: 'initialization' stamp: 'lr 6/8/2007 17:04'!
initializeApply: anApplyBlock conflictTest: aConflictTestBlock	
	applyBlock := anApplyBlock.
	conflictTestBlock := aConflictTestBlock! !

LRSTMChange subclass: #LRSTMObjectChange
	instanceVariableNames: 'original previous working'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Model'!

!LRSTMObjectChange class methodsFor: 'instance-creation' stamp: 'lr 6/8/2007 16:59'!
on: anObject
	^ self basicNew initializeOn: anObject! !

!LRSTMObjectChange methodsFor: 'actions' stamp: 'lr 6/8/2007 16:59'!
apply
	original restoreSnapshot: working! !

!LRSTMObjectChange methodsFor: 'testing' stamp: 'lr 6/8/2007 17:00'!
hasChanged
	^ (working isIdenticalToSnapshot: previous) not! !

!LRSTMObjectChange methodsFor: 'testing' stamp: 'lr 6/8/2007 17:00'!
hasConflict
	^ (original isIdenticalToSnapshot: previous) not! !

!LRSTMObjectChange methodsFor: 'initialization' stamp: 'lr 6/8/2007 16:59'!
initializeOn: anObject
	original := anObject.
	working := original snapshotCopy.
	previous := working snapshotCopy! !

!LRSTMObjectChange methodsFor: 'accessing' stamp: 'lr 6/8/2007 17:00'!
original
	^ original! !

!LRSTMObjectChange methodsFor: 'accessing' stamp: 'lr 6/8/2007 17:00'!
previous
	^ previous! !

!LRSTMObjectChange methodsFor: 'printing' stamp: 'lr 6/8/2007 17:01'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' ['; print: self original; nextPut: $]! !

!LRSTMObjectChange methodsFor: 'accessing' stamp: 'lr 6/8/2007 17:00'!
working
	^ working! !

Object subclass: #LRSTMHello
	instanceVariableNames: 'a b'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Tests'!

!LRSTMHello class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/29/2011 01:12'!
example
	|  p1 p2 hello |
	hello := self new.
	
	"Since this main thread is not running inside the OPSTMDomain, which would be easy, too,
	 but is not happening here, we need to manually move the object into the domain."
	
	hello a: 'main'; b: 'main'.
	
	p1 := [
		1 halt.
		[	hello a: 'p1'.
			Processor yield.
			hello b: 'p1'.] atomic
	] fork.

	p2 := [
		[	| b |
			b := hello b.
			Processor yield.
			hello b: 'p2'.] atomic
	] fork.! !

!LRSTMHello methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/29/2011 01:11'!
a
	^ a asString! !

!LRSTMHello methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/29/2011 01:12'!
a: obj
	a := obj asString! !

!LRSTMHello methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 16:54'!
b
	^ b asString! !

!LRSTMHello methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 16:54'!
b: obj
	b := obj asString! !

!LRSTMHello methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 11:34'!
initialize
	super initialize.
	a := #a.
	b := #b.! !

LRSTMHello subclass: #LRSTMHello2
	instanceVariableNames: 'c d'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Tests'!

!LRSTMHello2 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 11:36'!
c
	^ c! !

!LRSTMHello2 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 12:29'!
c: v
	c := v! !

!LRSTMHello2 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 11:36'!
d
	^ d! !

!LRSTMHello2 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 12:29'!
d: v
	d := v! !

!LRSTMHello2 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 11:33'!
initialize
	super initialize.
	c := #c.
	d := #d.! !

LRSTMHello2 subclass: #LRSTMHello3
	instanceVariableNames: 'e f'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Tests'!

!LRSTMHello3 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 11:36'!
a
	^ a! !

!LRSTMHello3 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 12:28'!
a: v
	a := v! !

!LRSTMHello3 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 11:36'!
b
	^ b! !

!LRSTMHello3 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 12:28'!
b: v
	b := v! !

!LRSTMHello3 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 11:36'!
c
	^ c! !

!LRSTMHello3 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 12:28'!
c: v
	c := v! !

!LRSTMHello3 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 11:36'!
d
	^ d! !

!LRSTMHello3 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 12:28'!
d: v
	d := v! !

!LRSTMHello3 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 11:36'!
e
	^ e! !

!LRSTMHello3 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 12:28'!
e: v
	e := v! !

!LRSTMHello3 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 11:36'!
f
	^ f! !

!LRSTMHello3 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 12:28'!
f: v
	f := v! !

!LRSTMHello3 methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/5/2011 11:33'!
initialize
	super initialize.
	e := #e.
	f := #f.! !

Object subclass: #LRSTMTransaction
	instanceVariableNames: 'parent context changes objectChanges'
	classVariableNames: ''
	poolDictionaries: 'ContextPart'
	category: 'OmniST-Languages-LRSTM-Model'!

!LRSTMTransaction class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/11/2011 22:01'!
initialize
	super initialize.
	
	(Process hasInstVarNamed: #currentTransaction) ifFalse: [
		Process addInstVarNamed: #currentTransaction.
	].! !

!LRSTMTransaction class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/11/2011 22:02'!
unload
	(Process hasInstVarNamed: #currentTransaction) ifTrue: [
		Process removeInstVarNamed: #currentTransaction.
	]! !

!LRSTMTransaction methodsFor: 'private' stamp: 'StefanMarr 12/11/2011 16:28'!
abort
	"Abort a transaction."
	
	<omniDoNotTransform>
	parent := context := changes := objectChanges := nil! !

!LRSTMTransaction methodsFor: 'public' stamp: 'StefanMarr 12/11/2011 16:29'!
abort: aValue
	"Abort a transaction."
	
	<omniDoNotTransform>
	Processor activeProcess currentTransaction: parent.
	thisContext swapSender: context.
	self abort.
	^ aValue! !

!LRSTMTransaction methodsFor: 'accessing' stamp: 'StefanMarr 12/11/2011 16:29'!
addChange: aChange
	<omniDoNotTransform>
	^ changes add: aChange! !

!LRSTMTransaction methodsFor: 'private' stamp: 'StefanMarr 12/11/2011 16:31'!
begin
	"Start a transaction. If this is a nested transaction copy the parent changes into our own context."
	
	<omniDoNotTransform>
	changes := OrderedCollection new.
	objectChanges := IdentityDictionary new.
	self isNested ifTrue: [
		parent objectChanges
			keysAndValuesDo: [ :key :value | objectChanges at: key put: value copy ] ]! !

!LRSTMTransaction methodsFor: 'accessing' stamp: 'StefanMarr 12/11/2011 16:32'!
changeFor: anObject
	<omniDoNotTransform>
	^ objectChanges at: anObject ifAbsentPut: [ self addChange: (LRSTMObjectChange on: anObject) ]! !

!LRSTMTransaction methodsFor: 'accessing' stamp: 'StefanMarr 12/11/2011 16:32'!
changes
	"Answer an ordered collection of unique changes."

	<omniDoNotTransform>
	^ changes! !

!LRSTMTransaction methodsFor: 'public' stamp: 'StefanMarr 12/11/2011 16:32'!
checkpoint
	"Checkpoint a transaction."

	<omniDoNotTransform>
	self commit; begin! !

!LRSTMTransaction methodsFor: 'private' stamp: 'StefanMarr 12/11/2011 16:32'!
commit
	"Commit a transaction."

	<omniDoNotTransform>
	self commitIfConflict: [ self signalConflict ]! !

!LRSTMTransaction methodsFor: 'private' stamp: 'StefanMarr 12/11/2011 16:32'!
commitIfConflict: aBlock
	"Commit a transaction atomically."
	
	<omniDoNotTransform>
	[	changes do: [ :each |
			each hasConflict
				ifTrue: [ ^ aBlock value ] ].
		self isNested 
			ifTrue: [
				objectChanges keysAndValuesDo: [ :key :value |
					(parent objectChanges includesKey: key)
						ifFalse: [ parent addChange: value ].
					parent objectChanges at: key put: value ] ]
			ifFalse: [ 	
				changes do: [ :each |
					each hasChanged
						ifTrue: [ each apply ] ] ] ]
		valueUnpreemptively! !

!LRSTMTransaction methodsFor: 'public' stamp: 'StefanMarr 12/11/2011 16:32'!
do: aBlock
	"Start a transaction with aBlock."

	<omniDoNotTransform>		
	^ self do: aBlock ifConflict: [ self signalConflict ]! !

!LRSTMTransaction methodsFor: 'public' stamp: 'StefanMarr 12/11/2011 22:18'!
do: aBlock ifConflict: aConflictBlock
	"Evaluate aBLock within a new transaction, unless we are already in an existing transactional context. Evaluate aConflictBlock if the transaction conflicts with concurrent edits."

	<omniDoNotTransform>
	| result |
	parent := Processor activeProcess currentTransaction.
	Processor activeProcess currentTransaction: self.
	context := thisContext sender.
	self begin.
	
	[[ result := aBlock 
		ensure: [ Processor activeProcess currentTransaction: parent ] ]
		ifCurtailed: [ self abort ].
	] omni: LRSTMDomain domain.
 
	self commitIfConflict: aConflictBlock.
	^ result! !

!LRSTMTransaction methodsFor: 'testing' stamp: 'StefanMarr 12/11/2011 16:33'!
isNested
	<omniDoNotTransform>
	^ parent notNil! !

!LRSTMTransaction methodsFor: 'accessing' stamp: 'StefanMarr 12/11/2011 16:33'!
objectChanges
	<omniDoNotTransform>
	^ objectChanges! !

!LRSTMTransaction methodsFor: 'accessing' stamp: 'StefanMarr 12/11/2011 16:33'!
parent
	<omniDoNotTransform>
	^ parent! !

!LRSTMTransaction methodsFor: 'public' stamp: 'StefanMarr 12/11/2011 16:35'!
retry: aBlock
	"Unconditionally retry to evaluate aBlock until there are no conflict."

	<omniDoNotTransform>
	^ self do: aBlock ifConflict: [ self retry: aBlock ]! !

!LRSTMTransaction methodsFor: 'private' stamp: 'StefanMarr 12/11/2011 16:35'!
signalConflict
	<omniDoNotTransform>
	^ LRSTMConflict new transaction: self; signal! !

Object subclass: #OActiveObjectMessage
	instanceVariableNames: 'promise methodToBeExecuted receiverObj arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-ActiveObject-Reflection'!

!OActiveObjectMessage class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/5/2011 17:17'!
method: aCompiledMethod on: anObj with: arguments
	| msg |
	msg := self new.
	msg method: aCompiledMethod.
	msg receiver: anObj.
	msg arguments: arguments.
	^ msg! !

!OActiveObjectMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/5/2011 23:17'!
arguments
	^ arguments! !

!OActiveObjectMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/5/2011 17:26'!
arguments: argArray
	arguments := argArray! !

!OActiveObjectMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/4/2011 17:23'!
initialize
	super initialize.
	promise := OPromise new.! !

!OActiveObjectMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/5/2011 23:17'!
method
	^ methodToBeExecuted! !

!OActiveObjectMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/5/2011 17:19'!
method: aCompiledMethod
	methodToBeExecuted := aCompiledMethod! !

!OActiveObjectMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/5/2011 23:17'!
receiver
	^ receiverObj! !

!OActiveObjectMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/5/2011 17:19'!
receiver: anObj
	receiverObj := anObj! !

!OActiveObjectMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/4/2011 17:24'!
resolve
	^ promise read.! !

!OActiveObjectMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/5/2011 23:19'!
result: anObj
	promise deliver: anObj! !

Object subclass: #OPActiveObject
	instanceVariableNames: 'omniDomain'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-ActiveObject'!

!OPActiveObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/5/2011 20:22'!
example
	"When instantianting an active object,
	 it does everything to be able to transform itself to a real active object.
	 Thus, all messages are handled asynchronously, and for that an event loop is
	 created, waiting on a certain message queue.
	 
	 Beyond that, we do not give any other interesting guarantees.
	 
	 Since we don't have anyother way of determining whether a method actually
	 returns a useful value, we use an explicit pragma:
	   <noReturnValue>
	 This indicates that the method does not return anything useful,
	 and the client does not block, but gets a self instead."! !

!OPActiveObject methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/5/2011 20:25'!
initialize
	omniDomain := OPActiveObjectDomain newFor: self.! !

OPActiveObject subclass: #OpAOPingPong
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-ActiveObject'!

!OpAOPingPong class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 00:47'!
example
	[ | ping |
		ping := OpAOPingPong new.
		ping ping. ] forkNamed: 'ping-pong-test'.! !

!OpAOPingPong methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 20:04'!
ping
	<noReturnValue>
	Transcript log: 'ping'.
	Transcript log: 'PingPong domain: ', omniDomain printString, '(', omniDomain hash asString, ')'.
	
	OpAOPingObj new printDomain.
	
	self pong.
	! !

!OpAOPingPong methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/5/2011 23:26'!
pong
	<noReturnValue>
	Transcript addEntry: 'pong'.! !

Object subclass: #OPAgent
	instanceVariableNames: 'omniDomain state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-Clojure'!

!OPAgent class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 03:07'!
example
	| agent |
	agent := OPAgent newWith: 1.

	[ 1 to: 10 do: [:i|
		agent send: [:old |
			Transcript log: '[agent-1] old: ', old asString, ' iteration: ', i asString.
			old + 1].
		(Delay forSeconds: 1) wait.
	]] forkNamed: 'agent-1'.

	[ 1 to: 10 do: [:i|
		agent send: [:old |
			Transcript log: '[agent-2] old: ', old asString, ' iteration: ', i asString.
			old + 1].
		(Delay forMilliseconds: 800) wait.
	]] forkNamed: 'agent-2'.

	[ 1 to: 20 do: [:i|
		agent send: [:old |
			Transcript log: '[agent-3] old: ', old asString, ' iteration: ', i asString.
			old + 1].
		(Delay forMilliseconds: 512) wait.
	]] forkNamed: 'agent-3'.

	Transcript log: 'Started all agent tester processes.'.! !

!OPAgent class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:55'!
newWith: state
	| a |
	a := self new.
	a state: state.
	^ a! !

!OPAgent methodsFor: 'initialization' stamp: 'StefanMarr 11/6/2011 02:48'!
initialize
	omniDomain := OPAgentDomain newFor: self.! !

!OPAgent methodsFor: 'agent' stamp: 'StefanMarr 11/6/2011 02:52'!
send: aBlock
	state := aBlock value: state! !

!OPAgent methodsFor: 'agent' stamp: 'StefanMarr 11/6/2011 02:52'!
send: aBlock with: anArgument
	state := aBlock value: state value: anArgument! !

!OPAgent methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:56'!
state: anObj
	state := anObj! !

Object subclass: #OPAgentMessage
	instanceVariableNames: 'methodToBeExecuted arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-Clojure-Reflection'!

!OPAgentMessage class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:33'!
method: aCompiledMethod with: arguments
	| msg |
	msg := self new.
	msg method: aCompiledMethod.
	msg arguments: arguments.
	^ msg! !

!OPAgentMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:33'!
arguments
	^ arguments! !

!OPAgentMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:32'!
arguments: argArray
	arguments := argArray! !

!OPAgentMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:32'!
method
	^ methodToBeExecuted! !

!OPAgentMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:32'!
method: aCompiledMethod
	methodToBeExecuted := aCompiledMethod! !

Object subclass: #OPAtom
	instanceVariableNames: 'omniDomain state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-Clojure'!

!OPAtom class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:03'!
example
	| atom |
	atom := OPAtom new.

	Transcript log: 'New atom: ', atom read asString.

	atom reset: 1.

	[1 to: 10 do: [:i |
		atom swap: [:old | 
			Transcript log: '[atom-1] old: ', old asString, ' in iteration: ', i asString.
			Processor yield.
			old + 1]
	]] forkNamed: 'atom-1'.

	[1 to: 10 do: [:i |
		atom swap: [:old | 
			Transcript log: '[atom-2] old: ', old asString, ' in iteration: ', i asString.
			Processor yield.
			old + 1]
	]] forkNamed: 'atom-2'.
! !

!OPAtom methodsFor: 'atom' stamp: 'StefanMarr 11/6/2011 01:43'!
compareTo: oldValue andSet: newValue
	"Clojure: (compare-and-set!! atom oldval newval)
	 Returns true on success, false otherwise."
	state = oldValue
		ifTrue:  [ state := newValue.
					 ^ true  ]
		ifFalse: [ ^ false ].! !

!OPAtom methodsFor: 'initialization' stamp: 'StefanMarr 11/6/2011 01:47'!
initialize
	omniDomain := OPAtomDomain newFor: self.! !

!OPAtom methodsFor: 'atom' stamp: 'StefanMarr 11/6/2011 01:43'!
read
	"Clojure deref/@ reads the most recent value"
	^ state! !

!OPAtom methodsFor: 'atom' stamp: 'StefanMarr 11/6/2011 01:45'!
reset: value
	state := value! !

!OPAtom methodsFor: 'atom' stamp: 'StefanMarr 11/6/2011 01:57'!
swap: updateBlock
	"Clojure: (swap!! atom f)
	 The block needs to take an argument,
	 and return a result value, like this:
	 [:oldVal | oldVal + 5]	"
	| currentState |
	[ | result |
	  currentState := state.
	  result := self compareTo: currentState
			andSet: (updateBlock value: currentState).
	  result ifFalse: [Transcript log: 'swap failed'].
	  result
		] whileFalse.
	
	! !

Object subclass: #OPSTMBuffer
	instanceVariableNames: 'map'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-STM-Misc'!

!OPSTMBuffer methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 22:53'!
checkConsistencyWithObjectsOtherwise: abortBlock
	"Execute abortBlock if buffer is not consistent with the referenced objects"
	| NoValue |
	NoValue := OPTransaction NoValue.
	map keysAndValuesDo: [:obj :shadow |
		shadow withIndexDo: [:val :idx |
			val ~= NoValue ifTrue: [
				"Ok, that is a value we read,
				 so it needs to be consistent with what we have currently."
				(obj instVarAt: idx) == val
					"Ooops, not consistent, so we abort here."
					ifFalse: [^abortBlock value.]
				]
			]
		].! !

!OPSTMBuffer methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 17:44'!
for: obj instVarAt: idx
	| shadow |
	shadow := map at: obj
					 ifAbsent: [^ OPTransaction NoValue].
	^ shadow at: idx.! !

!OPSTMBuffer methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 18:23'!
for: obj instVarAt: idx put: val
	| shadow |
	shadow := map at: obj
					 ifAbsentPut: [self newShadowFor: obj].
	^ shadow at: idx put: val.! !

!OPSTMBuffer methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 17:39'!
initialize
	super initialize.
	
	map := IdentityDictionary new.! !

!OPSTMBuffer methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 17:51'!
newShadowFor: obj
	| n shadow |
	n := obj class numberOfInstanceVariables.
	shadow := Array new: n.
	shadow atAllPut: OPTransaction NoValue.
	^ shadow! !

!OPSTMBuffer methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2011 18:41'!
performWrites
	| NoValue |
	NoValue := OPTransaction NoValue.
	map keysAndValuesDo: [:obj :shadow |
		shadow withIndexDo: [:val :idx |
			val ~= NoValue ifTrue: [
				"Ok, that is a value we wrote,
				 now lets write it for real"
				obj instVarAt: idx put: val.
			]
		]
	]! !

!Object methodsFor: '*OmniST-Languages-LRSTM-atomic' stamp: 'StefanMarr 12/11/2011 16:29'!
atomicAt: anInteger
	<omniDoNotTransform>
	^ self workingCopy at: anInteger! !

!Object methodsFor: '*OmniST-Languages-LRSTM-atomic' stamp: 'StefanMarr 12/11/2011 16:29'!
atomicAt: anInteger put: anObject
	<omniDoNotTransform>
	^ self workingCopy at: anInteger put: anObject! !

!Object methodsFor: '*OmniST-Languages-LRSTM-atomic' stamp: 'StefanMarr 12/11/2011 16:29'!
atomicBasicAt: anInteger
	<omniDoNotTransform>
	^ self workingCopy basicAt: anInteger! !

!Object methodsFor: '*OmniST-Languages-LRSTM-atomic' stamp: 'StefanMarr 12/11/2011 16:30'!
atomicBasicAt: anInteger put: anObject
	<omniDoNotTransform>
	^ self workingCopy basicAt: anInteger put: anObject! !

!Object methodsFor: '*OmniST-Languages-LRSTM-atomic' stamp: 'StefanMarr 12/11/2011 16:30'!
atomicInstVarAt: anInteger
	<omniDoNotTransform>
	^ self workingCopy instVarAt: anInteger! !

!Object methodsFor: '*OmniST-Languages-LRSTM-atomic' stamp: 'StefanMarr 12/11/2011 16:31'!
atomicInstVarAt: anInteger put: anObject
	<omniDoNotTransform>
	^ self workingCopy instVarAt: anInteger put: anObject! !

!Object methodsFor: '*OmniST-Languages-LRSTM-atomic' stamp: 'StefanMarr 12/11/2011 16:31'!
atomicShallowCopy
	<omniDoNotTransform>
	^ self workingCopy shallowCopy! !

!Object methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 6/7/2007 15:05'!
isIdenticalToSnapshot: anObject
	1 to: self class instSize do: [ :index | 
		(self instVarAt: index) == (anObject instVarAt: index)
			ifFalse: [ ^ false ] ].
	self class isVariable ifTrue: [
		1 to: self size do: [ :index | 
			(self at: index) == (anObject at: index)
				ifFalse: [ ^ false ] ] ].
	^ true! !

!Object methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 6/7/2007 13:47'!
restoreSnapshot: anObject
	<primitive: 168>
	self primitiveFailed! !

!Object methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 6/7/2007 13:48'!
snapshotCopy
	<primitive: 148>
	self primitiveFailed! !

!Object methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/30/2007 12:11'!
workingCopy
	"Answer a working copy to be used within the context of the current transaction."
	
	| transaction |
	transaction := Processor activeProcess currentTransaction
		ifNil: [ self error: 'No active transaction' ].
	^ (transaction changeFor: self) working! !

Object subclass: #OpAOPingObj
	instanceVariableNames: 'omniDomain'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-ActiveObject'!

!OpAOPingObj methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 19:59'!
printDomain
	Transcript log: omniDomain printString, '(', omniDomain hash asString, ')'.! !

Object subclass: #OstExperiments
	instanceVariableNames: 'omniDomain a b c d'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-POCs'!

!OstExperiments commentStamp: '<historical>' prior: 0!
This class had only a use while developing the compiler transformations.!

!OstExperiments methodsFor: 'nil' stamp: 'StefanMarr 12/7/2011 23:19'!
a: val
	a := val! !

!OstExperiments methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/7/2011 23:20'!
a: anA b: aB andSome: aC
	a := anA + aB + aC! !

!OstExperiments methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/7/2011 22:22'!
aAndB
	^ a + b! !

!OstExperiments methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/7/2011 23:20'!
abc
 | sd |
 a ifNil: [^b].
 a == b ifTrue: [^ a].
 ^ c! !

!OstExperiments methodsFor: 'nil' stamp: 'StefanMarr 12/8/2011 00:33'!
do: p1 a: p2 b: p3 c: p4 d: p5 e: p6
	^ p1 asString,
	  p2 asString,
	  p3 asString,
	  p4 asString,
	  p5 asString,
	  p6 asString
	  ! !

!OstExperiments methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/9/2011 13:44'!
manualTestChainedSend
	| result |
	
	result := [ 
		| x_omni_1_ |
		x_omni_1_ := 1 omniDomain requestExecutionOf: #bitOr: on: 1 with: 2.
		x_omni_1_ omniDomain requestExecutionOf: #bitOr: on: x_omni_1_ with: 4
	] fooBarEntryMessage.

	self omniDomain
		requestExecutionOf: #xx_omni_x_assert:equals:
		on: self
		with: 7
		with: result! !

!OstExperiments methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/8/2011 00:31'!
perform: sel a: p1 b: p2 c: p3 d: p4 e: p5 f: p6
	"dd"
	<primitive: 83>
	^ -1! !
LRSTMTransaction initialize!
LRSTMTest initialize!
OPTransaction initialize!
